@@ -34,13 +34,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
 
-DEFAULT_FONT='standard'
+DEFAULT_FONT = 'standard'
 
 
 def figlet_format(text, font=DEFAULT_FONT, **kwargs):
     fig = Figlet(font, **kwargs)
     return fig.renderText(text)
 
+
 def print_figlet(text, font=DEFAULT_FONT, **kwargs):
     print(figlet_format(text, font, **kwargs))
 
@@ -92,18 +93,24 @@ class FigletFont(object):
             fn = '%s.%s' % (font, extension)
             if pkg_resources.resource_exists('pyfiglet.fonts', fn):
                 data = pkg_resources.resource_string('pyfiglet.fonts', fn)
-                data = data.decode('ascii', 'replace')
+                data = data.decode('UTF-8', 'replace')
                 return data
         else:
             raise FontNotFound(font)
 
+    @classmethod
+    def isValidFont(cls, font):
+        if not font.endswith(('.flf', '.tlf')):
+            return False
+        f = pkg_resources.resource_stream('pyfiglet.fonts', font)
+        header = f.readline().decode('UTF-8', 'replace')
+        return cls.reMagicNumber.search(header)
+
     @classmethod
     def getFonts(cls):
         return [font.rsplit('.', 2)[0] for font
                 in pkg_resources.resource_listdir('pyfiglet', 'fonts')
-                if font.endswith(('.flf', '.tlf'))
-                   and cls.reMagicNumber.search(pkg_resources.resource_stream(
-                        'pyfiglet.fonts', font).readline().decode('ascii', 'replace'))]
+                if cls.isValidFont(font)]
 
     @classmethod
     def infoFont(cls, font, short=False):
@@ -112,13 +119,17 @@ class FigletFont(object):
         """
         data = FigletFont.preloadFont(font)
         infos = []
-        reMagicNumber = re.compile(r'^[tf]lf2.')
-        reStartMarker = re.compile(r'^(FONT|COMMENT|FONTNAME_REGISTRY|FAMILY_NAME|FOUNDRY|WEIGHT_NAME|SETWIDTH_NAME|SLANT|ADD_STYLE_NAME|PIXEL_SIZE|POINT_SIZE|RESOLUTION_X|RESOLUTION_Y|SPACING|AVERAGE_WIDTH|COMMENT|FONT_DESCENT|FONT_ASCENT|CAP_HEIGHT|X_HEIGHT|FACE_NAME|FULL_NAME|COPYRIGHT|_DEC_|DEFAULT_CHAR|NOTICE|RELATIVE_).*')
+        reStartMarker = re.compile(r"""
+            ^(FONT|COMMENT|FONTNAME_REGISTRY|FAMILY_NAME|FOUNDRY|WEIGHT_NAME|
+              SETWIDTH_NAME|SLANT|ADD_STYLE_NAME|PIXEL_SIZE|POINT_SIZE|
+              RESOLUTION_X|RESOLUTION_Y|SPACING|AVERAGE_WIDTH|COMMENT|
+              FONT_DESCENT|FONT_ASCENT|CAP_HEIGHT|X_HEIGHT|FACE_NAME|FULL_NAME|
+              COPYRIGHT|_DEC_|DEFAULT_CHAR|NOTICE|RELATIVE_).*""", re.VERBOSE)
         reEndMarker = re.compile(r'^.*[@#$]$')
-        for line in  data.splitlines()[0:100]:
-            if reMagicNumber.search(line) is None \
-               and reStartMarker.search(line) is None \
-               and reEndMarker.search(line) is None:
+        for line in data.splitlines()[0:100]:
+            if (cls.reMagicNumber.search(line) is None
+                    and reStartMarker.search(line) is None
+                    and reEndMarker.search(line) is None):
                 infos.append(line)
         return '\n'.join(infos) if not short else infos[0]
 
@@ -141,12 +152,15 @@ class FigletFont(object):
                 raise FontError('malformed header for %s' % self.font)
 
             hardBlank = header[0]
-            height, baseLine, maxLength, oldLayout, commentLines = map(int, header[1:6])
+            height, baseLine, maxLength, oldLayout, commentLines = map(
+                int, header[1:6])
             printDirection = fullLayout = None
 
             # these are all optional for backwards compat
-            if len(header) > 6: printDirection = int(header[6])
-            if len(header) > 7: fullLayout = int(header[7])
+            if len(header) > 6:
+                printDirection = int(header[6])
+            if len(header) > 7:
+                fullLayout = int(header[7])
 
             # if the new layout style isn't available,
             # convert old layout style. backwards compatability
@@ -185,7 +199,8 @@ class FigletFont(object):
 
                     line = end.sub('', line)
 
-                    if len(line) > width: width = len(line)
+                    if len(line) > width:
+                        width = len(line)
                     chars.append(line)
                 return width, chars
 
@@ -217,16 +232,51 @@ class FigletFont(object):
         return '<FigletFont object: %s>' % self.font
 
 
-class FigletString(str):
+unicode_string = type(''.encode('ascii').decode('ascii'))
+
+
+class FigletString(unicode_string):
     """
     Rendered figlet font
     """
 
     # translation map for reversing ascii art / -> \, etc.
-    __reverse_map__ = '\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !"#$%&\')(*+,-.\\0123456789:;>=<?@ABCDEFGHIJKLMNOPQRSTUVWXYZ]/[^_`abcdefghijklmnopqrstuvwxyz}|{~\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff'
+    __reverse_map__ = (
+        '\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f'
+        '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
+        ' !"#$%&\')(*+,-.\\'
+        '0123456789:;>=<?'
+        '@ABCDEFGHIJKLMNO'
+        'PQRSTUVWXYZ]/[^_'
+        '`abcdefghijklmno'
+        'pqrstuvwxyz}|{~\x7f'
+        '\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f'
+        '\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f'
+        '\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf'
+        '\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf'
+        '\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf'
+        '\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf'
+        '\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef'
+        '\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff')
 
     # translation map for flipping ascii art ^ -> v, etc.
-    __flip_map__ = '\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !"#$%&\'()*+,-.\\0123456789:;<=>?@VBCDEFGHIJKLWNObQbSTUAMXYZ[/]v-`aPcdefghijklwnopqrstu^mxyz{|}~\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff'
+    __flip_map__ = (
+        '\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f'
+        '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
+        ' !"#$%&\'()*+,-.\\'
+        '0123456789:;<=>?'
+        '@VBCDEFGHIJKLWNO'
+        'bQbSTUAMXYZ[/]v-'
+        '`aPcdefghijklwno'
+        'pqrstu^mxyz{|}~\x7f'
+        '\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f'
+        '\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f'
+        '\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf'
+        '\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf'
+        '\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf'
+        '\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf'
+        '\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef'
+        '\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff')
 
     def reverse(self):
         out = []
@@ -265,82 +315,100 @@ class FigletRenderingEngine(object):
         self.SM_KERN = 64
         self.SM_SMUSH = 128
 
-
     def smushChars(self, left='', right=''):
         """
         Given 2 characters which represent the edges rendered figlet
         fonts where they would touch, see if they can be smushed together.
         Returns None if this cannot or should not be done.
         """
-        if left.isspace() is True: return right
-        if right.isspace() is True: return left
+        if left.isspace() is True:
+            return right
+        if right.isspace() is True:
+            return left
 
-        # Disallows overlapping if previous or current char has a width of 1 or zero
-        if (self.prevCharWidth < 2) or (self.curCharWidth < 2): return
+        # Disallows overlapping if previous or current char has a width of 1 or
+        # zero
+        if (self.prevCharWidth < 2) or (self.curCharWidth < 2):
+            return
 
         # kerning only
-        if (self.base.Font.smushMode & self.SM_SMUSH) == 0: return
+        if (self.base.Font.smushMode & self.SM_SMUSH) == 0:
+            return
 
         # smushing by universal overlapping
         if (self.base.Font.smushMode & 63) == 0:
             # Ensure preference to visiable characters.
-            if left == self.base.Font.hardBlank: return right
-            if right == self.base.Font.hardBlank: return left
+            if left == self.base.Font.hardBlank:
+                return right
+            if right == self.base.Font.hardBlank:
+                return left
 
             # Ensures that the dominant (foreground)
             # fig-character for overlapping is the latter in the
             # user's text, not necessarily the rightmost character.
-            if self.base.direction == 'right-to-left': return left
-            else: return right
+            if self.base.direction == 'right-to-left':
+                return left
+            else:
+                return right
 
         if self.base.Font.smushMode & self.SM_HARDBLANK:
-            if left == self.base.Font.hardBlank and right == self.base.Font.hardBlank:
+            if (left == self.base.Font.hardBlank
+                    and right == self.base.Font.hardBlank):
                 return left
 
-        if left == self.base.Font.hardBlank or right == self.base.Font.hardBlank:
+        if (left == self.base.Font.hardBlank
+                or right == self.base.Font.hardBlank):
             return
 
         if self.base.Font.smushMode & self.SM_EQUAL:
             if left == right:
                 return left
 
+        smushes = ()
+
         if self.base.Font.smushMode & self.SM_LOWLINE:
-            if (left  == '_') and (right in r'|/\[]{}()<>'): return right
-            if (right == '_') and (left  in r'|/\[]{}()<>'): return left
+            smushes += (('_', r'|/\[]{}()<>'),)
 
         if self.base.Font.smushMode & self.SM_HIERARCHY:
-            if (left  == '|')   and (right in r'|/\[]{}()<>'): return right
-            if (right == '|')   and (left  in r'|/\[]{}()<>'): return left
-            if (left  in r'\/') and (right in '[]{}()<>'): return right
-            if (right in r'\/') and (left  in '[]{}()<>'): return left
-            if (left  in '[]')  and (right in '{}()<>'): return right
-            if (right in '[]')  and (left  in '{}()<>'): return left
-            if (left  in '{}')  and (right in '()<>'): return right
-            if (right in '{}')  and (left  in '()<>'): return left
-            if (left  in '()')  and (right in '<>'): return right
-            if (right in '()')  and (left  in '<>'): return left
+            smushes += (
+                ('|', r'|/\[]{}()<>'),
+                (r'\/', '[]{}()<>'),
+                ('[]', '{}()<>'),
+                ('{}', '()<>'),
+                ('()', '<>'),
+            )
+
+        for a, b in smushes:
+            if left in a and right in b:
+                return right
+            if right in a and left in b:
+                return left
 
         if self.base.Font.smushMode & self.SM_PAIR:
             for pair in [left+right, right+left]:
-                if pair in ['[]', '{}', '()']: return '|'
+                if pair in ['[]', '{}', '()']:
+                    return '|'
 
         if self.base.Font.smushMode & self.SM_BIGX:
-            if (left == '/') and (right == '\\'): return '|'
-            if (right == '/') and (left == '\\'): return 'Y'
-            if (left == '>') and (right == '<'): return 'X'
-
+            if (left == '/') and (right == '\\'):
+                return '|'
+            if (right == '/') and (left == '\\'):
+                return 'Y'
+            if (left == '>') and (right == '<'):
+                return 'X'
         return
 
     def smushAmount(self, left=None, right=None, buffer=[], curChar=[]):
         """
-        Calculate the amount of smushing we can do between this char and the last
-        If this is the first char it will throw a series of exceptions which
-        are caught and cause appropriate values to be set for later.
+        Calculate the amount of smushing we can do between this char and the
+        last If this is the first char it will throw a series of exceptions
+        which are caught and cause appropriate values to be set for later.
 
         This differs from C figlet which will just get bogus values from
         memory and then discard them after.
         """
-        if (self.base.Font.smushMode & (self.SM_SMUSH | self.SM_KERN)) == 0: return 0
+        if (self.base.Font.smushMode & (self.SM_SMUSH | self.SM_KERN)) == 0:
+            return 0
 
         maxSmush = self.curCharWidth
         for row in range(0, self.base.Font.height):
@@ -370,7 +438,8 @@ class FigletRenderingEngine(object):
 
             if ch1 == '' or ch1 == ' ':
                 amt += 1
-            elif ch2 != '' and self.smushChars(left=ch1, right=ch2) is not None:
+            elif (ch2 != ''
+                    and self.smushChars(left=ch1, right=ch2) is not None):
                 amt += 1
 
             if amt < maxSmush:
@@ -386,7 +455,8 @@ class FigletRenderingEngine(object):
         buffer = ['' for i in range(self.base.Font.height)]
 
         for c in map(ord, list(text)):
-            if not c in self.base.Font.chars: continue
+            if c not in self.base.Font.chars:
+                continue
             curChar = self.base.Font.chars[c]
             self.curCharWidth = self.base.Font.width[c]
             maxSmush = self.smushAmount(buffer=buffer, curChar=curChar)
@@ -421,16 +491,19 @@ class FigletRenderingEngine(object):
 
             self.prevCharWidth = self.curCharWidth
 
-
         # Justify text. This does not use str.rjust/str.center
         # specifically because the output would not match FIGlet
         if self.base.justify == 'right':
             for row in range(0, self.base.Font.height):
-                buffer[row] = (' ' * (self.base.width - len(buffer[row]) - 1)) + buffer[row]
+                buffer[row] = (
+                    ' ' * (self.base.width - len(buffer[row]) - 1)
+                ) + buffer[row]
 
         elif self.base.justify == 'center':
             for row in range(0, self.base.Font.height):
-                buffer[row] = (' ' * int((self.base.width - len(buffer[row])) / 2)) + buffer[row]
+                buffer[row] = (
+                    ' ' * int((self.base.width - len(buffer[row])) / 2)
+                ) + buffer[row]
 
         # return rendered ASCII with hardblanks replaced
         buffer = '\n'.join(buffer) + '\n'
@@ -495,19 +568,31 @@ class Figlet(object):
 
 
 def main():
-    parser = OptionParser(version=__version__, usage='%prog [options] [text..]')
+    parser = OptionParser(version=__version__,
+                          usage='%prog [options] [text..]')
     parser.add_option('-f', '--font', default=DEFAULT_FONT,
-            help='font to render with (default: %default)', metavar='FONT')
-    parser.add_option('-D', '--direction', type='choice', choices=('auto', 'left-to-right', 'right-to-left'),
-            default='auto', metavar='DIRECTION', help='set direction text will be formatted in (default: %default)')
-    parser.add_option('-j', '--justify', type='choice', choices=('auto', 'left', 'center', 'right'), default='auto',
-            metavar='SIDE', help='set justification, defaults to print direction')
+                      help='font to render with (default: %default)',
+                      metavar='FONT')
+    parser.add_option('-D', '--direction', type='choice',
+                      choices=('auto', 'left-to-right', 'right-to-left'),
+                      default='auto', metavar='DIRECTION',
+                      help='set direction text will be formatted in '
+                           '(default: %default)')
+    parser.add_option('-j', '--justify', type='choice',
+                      choices=('auto', 'left', 'center', 'right'),
+                      default='auto', metavar='SIDE',
+                      help='set justification, defaults to print direction')
     parser.add_option('-w', '--width', type='int', default=80, metavar='COLS',
-            help='set terminal width for wrapping/justification (default: %default)' )
-    parser.add_option('-r', '--reverse', action='store_true', default=False, help='shows mirror image of output text')
-    parser.add_option('-F', '--flip', action='store_true', default=False, help='flips rendered output text over')
-    parser.add_option('-l', '--list_fonts', action='store_true', default=False, help='show installed fonts list')
-    parser.add_option('-i', '--info_font', action='store_true', default=False, help='show font\'s information, use with -f FONT')
+                      help='set terminal width for wrapping/justification '
+                           '(default: %default)')
+    parser.add_option('-r', '--reverse', action='store_true', default=False,
+                      help='shows mirror image of output text')
+    parser.add_option('-F', '--flip', action='store_true', default=False,
+                      help='flips rendered output text over')
+    parser.add_option('-l', '--list_fonts', action='store_true', default=False,
+                      help='show installed fonts list')
+    parser.add_option('-i', '--info_font', action='store_true', default=False,
+                      help='show font\'s information, use with -f FONT')
     opts, args = parser.parse_args()
 
     if opts.list_fonts:
@@ -530,11 +615,18 @@ def main():
     )
 
     r = f.renderText(text)
-    if opts.reverse is True: r = r.reverse()
-    if opts.flip is True: r = r.flip()
-    print(r)
+    if opts.reverse:
+        r = r.reverse()
+    if opts.flip:
+        r = r.flip()
 
+    if sys.version_info > (3,):
+        # Set stdout to binary mode
+        sys.stdout = sys.stdout.detach()
+
+    sys.stdout.write((r + '\n').encode('UTF-8'))
     return 0
 
+
 if __name__ == '__main__':
     sys.exit(main())
